function [x, y ,out] = cont_framework(opts,solver)
%CONT_FRAMEOWRK Continuation framework for pdNCG_ls, pdNCG_lg, pdNCG_user,
%               pdNCG_TV, pdNCG_W.

tic; % start counting time.

%Initialize total number of pdNCG iterations.
total_itr = 0;

% initialize data.
order_mu  = max(-floor(log(abs(opts.mu))./log(10)),0);
order_tau = max(-floor(log(abs(opts.tau))./log(10)),0);
    
t_mu  = max(order_mu-1,0);
t_tau = max(order_tau-1,0);

if t_mu == 0
    mus_vector = ones(opts.cont_steps,1).*opts.mu;
else
    mus_vector = logspace(log10(0.1),log10(opts.mu),opts.cont_steps);
end

if t_tau == 0
    taus_vector = ones(opts.cont_steps,1).*opts.tau;
else
    taus_vector = logspace(log10(0.1),log10(opts.tau),opts.cont_steps);
end

% Begin continuation.
for i=1:opts.cont_steps
    
    opts.mu  = mus_vector(i);  
    opts.tau = taus_vector(i);
    
    if opts.verbose >= 1
        fprintf('%5s:  %d \n','Total continuation steps',opts.cont_steps);
        fprintf('%5s: %d \n','Current continuation step',i);   
        fprintf('%5s: %d \n','Current mu ',opts.mu);
        fprintf('%5s: %d \n','Current tau',opts.tau); 
    end
    
    [x, y, out] = solver(opts);
    opts.x0 = x; 
    opts.y0 = y;
    
    total_itr = total_itr + out.itr;
end

out.itr = total_itr;

% Stop measuring time and save total time.
out.time = toc;